"""The EtcKeeper helper module."""
from subprocess import check_call

from charmhelpers.core import hookenv, host, templating, unitdata

from charms.reactive.helpers import any_file_changed


class EtcKeeper:
    """The EtcKeeper helper class."""

    config_file = "/etc/etckeeper/etckeeper.conf"

    def __init__(self):
        """Instantiate class variables."""
        self.charm_config = hookenv.config()
        self.kv = unitdata.kv()

    @property
    def is_initialized(self):
        """Verify etckeeper status."""
        return self.kv.get("etckeeper_init")

    def run_etckeeper(self, args):
        """Run etckeeper with the provided arguments."""
        if len(args) > 0:
            cmd = ["etckeeper"]
            cmd.extend(args)
            return check_call(cmd)
        return False

    def render_config(self):
        """Render configuration file."""
        hookenv.log(
            "Rendering etckeeper configuration to {}".format(self.config_file),
            hookenv.DEBUG,
        )
        templating.render(
            "etckeeper.conf.j2",
            self.config_file,
            {
                "etckeeper_remote": self.charm_config.get("vcs-remote"),
                "etckeeper_vcs": self.charm_config.get("vcs"),
            },
        )
        if any_file_changed([self.config_file]):
            return self.init_repo()
        return False

    def init_repo(self):
        """Initialise the etckeeper repository."""
        if self.run_etckeeper(["init"]):
            self.kv.set("etckeeper_init", True)
            return True
        else:
            return False

    def bootstrap_etc(self):
        """Set up repo and update KV with status."""
        if self.is_initialized:
            hookenv.log(
                "Skipping etckeeper init on already-initialised repo.", hookenv.DEBUG
            )
            return True
        if self.init_repo():
            return True
        return False

    def enable_timer(self):
        """Enable daily timer for etckeeper."""
        host.service("enable", "etckeeper.timer")

    def disable_timer(self):
        """Disable daily timer for etckeeper."""
        host.service("disable", "etckeeper.timer")

    def configure(self):
        """Configure etckeeper."""
        self.render_config()
        self.bootstrap_etc()
        if self.charm_config.get("daily"):
            self.enable_timer()
        else:
            self.disable_timer()

    def commit(self):
        """Commit changes and push to a remote."""
        self.run_etckeeper(["commit", "Charm initiated commit"])
        return True
