#!/usr/bin/python3
"""Unit tests for the etckeeper charm."""
import mock

import pytest


# If layer options are used, add this to etckeeper
# and import layer in lib_charm_etckeeper
@pytest.fixture
def mock_layers(monkeypatch):
    """Mock options for included layers."""
    import sys

    sys.modules["charms.layer"] = mock.Mock()
    sys.modules["reactive"] = mock.Mock()
    # Mock any functions in layers that need to be mocked here

    def options(layer):
        # mock options for layers here
        if layer == "example-layer":
            options = {"port": 9999}
            return options
        else:
            return None

    monkeypatch.setattr("lib_charm_etckeeper.layer.options", options)


@pytest.fixture
def mock_hookenv_config(monkeypatch):
    """Mock hookenv.config."""
    import yaml

    def mock_config():
        cfg = {}
        yml = yaml.load(open("./config.yaml"))

        # Load all defaults
        for key, value in yml["options"].items():
            cfg[key] = value["default"]

        # Manually add cfg from other layers
        # cfg['my-other-layer'] = 'mock'
        return cfg

    monkeypatch.setattr("lib_charm_etckeeper.hookenv.config", mock_config)


@pytest.fixture
def mock_remote_unit(monkeypatch):
    """Mock the details of a remote related unit."""
    monkeypatch.setattr(
        "lib_charm_etckeeper.hookenv.remote_unit", lambda: "unit-mock/0"
    )


@pytest.fixture
def mock_charm_dir(monkeypatch):
    """Mock the charm working directory."""
    monkeypatch.setattr("lib_charm_etckeeper.hookenv.charm_dir", lambda: ".")


@pytest.fixture
def mock_template(monkeypatch):
    """Mock syscalls used in the tempating library to prevent permissions problems."""
    monkeypatch.setattr("lib_charm_etckeeper.templating.host.os.fchown", mock.Mock())
    monkeypatch.setattr("lib_charm_etckeeper.templating.host.os.chown", mock.Mock())
    monkeypatch.setattr("lib_charm_etckeeper.templating.host.os.fchmod", mock.Mock())


@pytest.fixture
def mock_host_service(monkeypatch):
    """Mock host import on lib_charm_etckeeper."""

    def mocked_service(action, name):
        print("Called {} on {}".format(action, name))
        return True

    mock_service = mock.Mock()
    mock_service.side_effect = mocked_service
    monkeypatch.setattr("lib_charm_etckeeper.host.service", mock_service)
    return mock_service


@pytest.fixture
def mock_check_call(monkeypatch):
    """Mock subprocess check_call on lib_check_etckeeper."""

    def mocked_check_call(*args, **kwargs):
        return True

    mock_call = mock.Mock()
    mock_call.side_effect = mocked_check_call
    monkeypatch.setattr("lib_charm_etckeeper.check_call", mock_call)
    return mock_call


@pytest.fixture
def etckeeper(
    tmpdir,
    mock_hookenv_config,
    mock_charm_dir,
    mock_check_call,
    mock_host_service,
    mock_template,
    monkeypatch,
):
    """Mock the charm helper library."""
    from lib_charm_etckeeper import EtcKeeper

    helper = EtcKeeper()

    # Patch vars here
    config_file = tmpdir.join("etckeeper.conf")
    helper.config_file = config_file

    # Any other functions that load helper will get this version
    monkeypatch.setattr("lib_charm_etckeeper.EtcKeeper", lambda: helper)

    return helper
